@@ -14,7 +14,7 @@ use Text::CSV::Separator qw(get_separator);
 use Readonly;
 Readonly my $YARKONI_MAX => 20;
 
-$Lingua::Norms::SUBTLEX::VERSION = '0.04';
+$Lingua::Norms::SUBTLEX::VERSION = '0.05';
 
 =head1 NAME
 
@@ -22,13 +22,13 @@ Lingua::Norms::SUBTLEX - Retrieve frequency values and frequency-based lists for
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =head1 SYNOPSIS
 
  use feature qw(say);
  use Lingua::Norms::SUBTLEX;
- my $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'UK'); # or US, NL
+ my $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'US'); # or NL, UK, DE
  my $bool = $subtlex->is_normed(string => 'fuip'); # isa_word ? 
  my $frq = $subtlex->frq_opm(string => 'frog'); # freq. per million, or get log/zipf
  my $href = $subtlex->freqhash(words => [qw/frog fish ape/]); # freqs. for a list of words
@@ -46,9 +46,9 @@ Version 0.04
 
 =head1 DESCRIPTION
 
-The module facilitates access to raw data and descriptive statistics on word-frequency and parts-of-speech, as provided in the SUBTLEX-US, SUBTLEX-UK and SUBTLEX-NL databases (see L<REFERENCES|Lingua::Norms::SUBTLEX/REFERENCES>). For example, the SUBTLEX-US database is based on a study of 74,286 letter-strings, with frequencies of occurrence within a corpus of some 30 million words from the subtitles of 8,388 film and television episodes. The frequency data obtained in this way have been shown to offer more psychologically predictive measures than those derived from books, newsgroup posts, and similar.
+The module facilitates access to raw data and descriptive statistics on word-frequency and parts-of-speech, as provided in the SUBTLEX-DE, SUBTLEX-NL, SUBTLEX-UK and SUBTLEX-US databases (see L<REFERENCES|Lingua::Norms::SUBTLEX/REFERENCES>). For example, the SUBTLEX-US database is based on a study of 74,286 letter-strings, with frequencies of occurrence within a corpus of some 30 million words from the subtitles of 8,388 film and television episodes. The frequency data obtained in this way have been shown to offer more psychologically predictive measures than those derived from books, newsgroup posts, and similar.
 
-There are three groups of retrievable stats and sampling rules: (1) frequency; (2)contextual diversity (number of films/episodes appeared in); and (3) parts-of-speech. Depending on the source language, frequency is given as a count (L<frq_count|Lingua::Norms::SUBTLEX/frq_count>), occurrences per million (frq_opm), logarithm of the opm (frq_log), and/or 7-point scaled (L<frq_zipf|Lingua::Norms::SUBTLEX/frq_zipf>); contextual diversity is given as a count (cd_count), a percentage (cd_pct), or a logarithm (cd_log). For parts-of-speech, L<pos|Lingua::Norms::SUBTLEX/pos> returns a string giving the dominant part. Sampling is given by the same labels, with keys with min/max values (or a whitelist of acceptable parts-of-speech).
+There are three groups of retrievable stats and sampling rules: (1) frequency; (2)contextual diversity (number of films/episodes appeared in); and (3) parts-of-speech. Depending on the source language, frequency is given as a count (L<frq_count|Lingua::Norms::SUBTLEX/frq_count>), occurrences per million (L<frq_opm|Lingua::Norms::SUBTLEX/frq_opm>), logarithm of the opm (L<frq_log|Lingua::Norms::SUBTLEX/frq_log>), and/or 7-point scaled (L<frq_zipf|Lingua::Norms::SUBTLEX/frq_zipf>); contextual diversity is given as a count (L<cd_count|Lingua::Norms::SUBTLEX/cd_count>), a percentage (L<cd_pct|Lingua::Norms::SUBTLEX/cd_pct>), or a logarithm (L<cd_log|Lingua::Norms::SUBTLEX/cd_log>). For parts-of-speech, L<pos|Lingua::Norms::SUBTLEX/pos> returns a string giving the dominant part. Sampling is given by the same labels, with keys with min/max values (or a whitelist of acceptable parts-of-speech).
 
 A small sample from each of the databases is included in the installation distribution for testing purposes. The complete files need to be downloaded via the following URLs. The local directory location or actual pathname of these files can be given in class construction (by the arguments B<dir> and B<path>); otherwise the default location--the directory "SUBTLEX" alongside the module itself in the locally configured Perl sitelib--will be used, and the correct file determined by  inclusion of B<lang> value within its filename. The filenames of the original files downloaded from the following sites are supported in this way, and it does not matter if (as varies between the files) the fields are comma-separated or tab-delimited.
 
@@ -66,7 +66,11 @@ For the B<British> norms, install the file "SUBTLEX-UK.txt" from within the "SUB
 
 =item SUBTLEX-NL
 
-For the B<Dutch> norms, install the file "SUBTLEX-NL.with-pos.txt" from within the archive "SUBTLEX-NL.with-pos.txt.zip" via L<crr.ugent.be/programs-data/subtitle-frequencies/subtlex-nl|http://crr.ugent.be/programs-data/subtitle-frequencies/subtlex-nl>. This database does not define a value for zipf frequency, so the "zipf" method will return an empty string if called with NL as the "lang".
+For the B<Dutch> norms, install the file "SUBTLEX-NL.with-pos.txt" from within the archive "SUBTLEX-NL.with-pos.txt.zip" via L<crr.ugent.be|http://crr.ugent.be/programs-data/subtitle-frequencies/subtlex-nl>. This database does not define a value for Zipf frequency, so the "zipf" method will return an empty string if called with NL as the "lang".
+
+=item SUBTLEX-DE
+
+For the B<German> norms, dowload the file "SUBTLEX-DE_cleaned_with_Google00.txt" via L<crr.ugent.be|http://crr.ugent.be/archives/534>. There is no CD, POS or Zipf data at this point, so only the "frq_" methods, and the "on_" methods (based on realtime calculation work with this language. The file contains other information, including Google-based frequencies, for which this module does not provide retrieval at this time.
 
 =back
 
@@ -78,11 +82,11 @@ All methods are called via the class object, and with named (hash of) arguments,
 
 =head2 new
 
- $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'US'); # or 'UK', 'NL' - looking in Perl sitelib
+ $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'US'); # or 'UK', 'NL', 'DE' - looking in Perl sitelib
  $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'US', dir => 'file_location'); # where to look
  $subtlex = Lingua::Norms::SUBTLEX->new(lang => 'US', path => 'actual_file');
 
-Returns a class object for accessing other methods. The parameter B<lang> should be set to specify the particular language database: US, UK or NL (Dutch); otherwise US (being the first published in the series) is the default. Optional arguments B<dir> or B<path> can be given to specify the location or actual file (respectively) of the database. The default location is within the "Lingua/Norms/SUBTLEX" directory within the 'sitelib' configured for the local Perl installation (as per L<Config.pm|Config>). The method will C<croak> if the given B<path> or default location cannot be found.
+Returns a class object for accessing other methods. The parameter B<lang> should be set to specify the particular language database: DE (German), NL (Dutch), UK (British) or US (American); otherwise US (being the first published in the series) is the default. Optional arguments B<dir> or B<path> can be given to specify the location or actual file (respectively) of the database. The default location is within the "Lingua/Norms/SUBTLEX" directory within the 'sitelib' configured for the local Perl installation (as per L<Config.pm|Config>). The method will C<croak> if the given B<path> or default location cannot be found.
 
 =cut
 
@@ -122,7 +126,9 @@ sub new {
 
     # identify needed field indices within this file:
     # - arg 'fieldpath' only used for testing, assumed not useful for user:
-    my %fields = ( US => 1, UK => 2, NL => 3 );
+    my %fields = ( US => 1, UK => 2, NL => 3, DE => 4 );
+    croak 'Cannot determine fields for given language'
+      if nocontent( $fields{$lang} );
     my $fieldpath =
         $args{'fieldpath'}
       ? $args{'fieldpath'}
@@ -160,8 +166,8 @@ sub is_normed {
     open my $fh, q{<}, $self->{'path'} or croak $OS_ERROR;
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip headings
-        /^([^,]+)/msx
-          or next; # isolate first token ahead of a comma as $1 in this csv file
+        /^([^\Q$self->{'delim'}\E]+)/msx
+          or next;   # isolate first token ahead of delimiter as $1 in this file
         if ( $str eq $1 ) {    # first token equals given string?
             $res = 1;          # set result to return as true
             last;              # got it, so abort look-up
@@ -302,13 +308,13 @@ sub frq_hash {
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;
         chomp;
-        my @line = split m/$self->{'delim'}/msx;
+        my @line = split m/\Q$self->{'delim'}\E/msx;
         if ( hascontent( $frq{ lc( $line[0] ) } ) ) {
             $frq{ lc( $line[0] ) }->[0] = $line[$col_i];
         }
     }
     close $fh or croak $OS_ERROR;
-    return { map { $_->[1] => $_->[0] }
+    return { map { $_->[1] => _nummify($_->[0]) }
           values %frq };    # assign values to original case strings
 }
 *freqhash = \&frq_hash;
@@ -329,10 +335,10 @@ sub pos {
     open my $fh, q{<}, $self->{'path'} or croak "$OS_ERROR\n";
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/msx;
+        /^([^\Q$self->{'delim'}\E]+)/msx;
         if ( $word eq $1 ) {
             chomp;
-            my @line = split m/$self->{'delim'}/msx, $_;
+            my @line = split m/\Q$self->{'delim'}\E/msx, $_;
             $pos = $line[ $self->{'IDX'}->{'pos'} ];
             last;
         }
@@ -410,7 +416,7 @@ sub on_count {
     open my $fh, q{<}, $self->{'path'} or die "$OS_ERROR\n";
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/msx or next;
+        /^([^\Q$self->{'delim'}\E]+)/msx or next;
         my $test = lc($1);
         if ( $ortho->are_orthons( $word, $test ) ) {
             push @orthons, $test;
@@ -503,7 +509,7 @@ sub on_frq_zipf_mean {
 
 I<Alias>: ldist
 
-Returns the mean L<Levenshtein Distance|http://www.let.rug.nl/%7Ekleiweg/lev/levenshtein.html> from a letter-string to its B<lim> closest orthographic neighbours. The default B<lim>it is 20, as defined in Yarkoni et al. (2008). The module uses the matrix-based calculation of Levenshtein Distance as implemented in this author's L<Lingua::Orthon|Lingua::Orthon> module. No defined value is returned if no Levenshtein Distance is found (whereas zero would connote "identical to everything").
+Returns the mean L<Levenshtein Distance|http://www.let.rug.nl/%7Ekleiweg/lev/levenshtein.html> from a letter-string to its B<lim> closest orthographic neighbours. The default B<lim>it is 20, as defined in Yarkoni et al. (2008). The module uses the matrix-based calculation of Levenshtein Distance as implemented in L<Lingua::Orthon|Lingua::Orthon> module. No defined value is returned if no Levenshtein Distance is found (whereas zero would connote "identical to everything").
 
 =cut
 
@@ -511,7 +517,8 @@ sub on_ldist {
     my ( $self, %args ) = @_;
     croak 'No string to test' if !$args{'string'};
     $args{'lim'} ||= $YARKONI_MAX;
-    return _get_orthon_ldist( $args{'string'}, $self->{'path'}, $args{'lim'} );
+    return _get_orthon_ldist( $args{'string'}, $self->{'path'}, $args{'lim'},
+        $self->{'delim'} );
 }
 *ldist = \&on_ldist;
 
@@ -530,9 +537,9 @@ For the minimum/maximum constrained criteria, the two limits are given as a refe
 
 The consonant-vowel pattern is specified as a string by the usual convention, e.g., 'CCVCC' defines a 5-letter word starting and ending with pairs of consonants, the pairs separated by a vowel. 'Y' is defined here as a consonant.
 
-A finer selection of particular letters can be made by giving a regular expression as a string to the B<regex> argument. In the example above, only letter-strings starting with the letter 'f', followed by one of more other letters, are specified. Alternatively, for example, '[^aeiouy]$' specifies that the letter-strings must not end with a vowel (here including 'y'). The entire example for '^f', including the shown arguments for B<cv_pattern>, B<freq>, B<onc> and B<length>, would return only two words: I<fiji> and I<fuse>.
+A finer selection of particular letters can be made by giving a regular expression as a string to the B<regex> argument. In the example above, only letter-strings starting with the letter 'f', followed by one of more other letters, are specified. Alternatively, for example, '[^aeiouy]$' specifies that the letter-strings must not end with a vowel (here including 'y'). The entire example for '^f', including the shown arguments for B<cv_pattern>, B<freq>, B<onc> and B<length>, would return only two words: I<fiji> and I<fuse> from SUBTLEX-US.
 
-The selection procedure will be made particularly slow wherever B<onc> is specified (as this has to be calculated in real-time) and no arguments are given for B<cv_pattern> and C<regex> (which are tested ahead of any other criteria).
+The selection procedure will be made particularly slow wherever B<onc> is specified (as this has to be calculated in real-time) and no arguments are given for B<cv_pattern> and B<regex> (which are tested ahead of any other criteria).
 
 Syllable-counts might be added in future; existing algorithms in the Lingua family are not sufficiently reliable for the purposes to which the present module might often be put; an alternative is being worked on.
 
@@ -561,7 +568,7 @@ sub list_strings {
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
         chomp;
-        my @line = split m/$self->{'delim'}/msx;
+        my @line = split m/\Q$self->{'delim'}\E/msx;
         next if !_in_range( length( $line[0] ), @{ $args{'length'} } );
         for ( keys %patterns ) {
             next LINES if $line[0] !~ $patterns{$_};
@@ -569,7 +576,7 @@ sub list_strings {
         for (qw/freq opm frq_opm frq_log log frq_zipf zipf cd_pct cd_log/) {
             if ( ref $args{$_} ) {
                 next LINES
-                  if !_in_range( $line[ $self->{'IDX'}->{$_} ],
+                  if !_in_range( _nummify($line[ $self->{'IDX'}->{$_} ]),
                     @{ $args{$_} } );
             }
         }
@@ -605,7 +612,7 @@ sub all_strings {
     open my $fh, q{<}, $self->{'path'} or croak $OS_ERROR;
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/msx or next;
+        /^([^\Q$self->{'delim'}\E]+)/msx or next;
         push @list, $1;
     }
     close $fh or croak;
@@ -632,7 +639,7 @@ sub random_string {
       File::RandomLine->new( $self->{'path'}, { algorithm => 'uniform' } );
     my @ari = ();
     while ( not scalar @ari or $ari[0] eq 'Word' ) {
-        @ari = split m/$self->{'delim'}/msx, $rl->next;
+        @ari = split m/\Q$self->{'delim'}\E/msx, $rl->next;
     }
     return wantarray ? @ari : $ari[0];
 }
@@ -671,12 +678,12 @@ sub _get_orthon_f {
     open( my $fh, q{<}, $path ) or croak $OS_ERROR;
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/xsm or next;              # capture first token
+        /^([^\Q$delim\E]+)/xsm or next;     # capture first token
         my $test = lc($1);
         if ( $ortho->are_orthons( $word, $test ) ) {    # Lingua::Orthon method
             chomp;
-            my @line = split m/$delim/xsm;
-            push @freqs, $line[$idx];
+            my @line = split m/\Q$delim\E/xsm;
+            push @freqs, _nummify($line[$idx]);
         }
     }
     close $fh or croak $OS_ERROR;
@@ -684,7 +691,7 @@ sub _get_orthon_f {
 }
 
 sub _get_orthon_ldist {
-    my ( $str, $path, $lim ) = @_;
+    my ( $str, $path, $lim, $delim ) = @_;
     my $word = lc($str);
     require Lingua::Orthon;
     my $ortho  = Lingua::Orthon->new();
@@ -693,14 +700,14 @@ sub _get_orthon_ldist {
     my @tests = ();
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/msx or next;
+        /^([^\Q$delim\E]+)/msx or next;
         my $test = lc($1);
         next if $word eq $test;
         push @ldists, $ortho->ldist( $word, $test );
     }
     close $fh or croak $OS_ERROR;
     my @sorted = sort { $a <=> $b } @ldists;
-    return mean(@sorted[ 0 .. $lim - 1 ]);
+    return mean( @sorted[ 0 .. $lim - 1 ] );
 }
 
 sub _get_fieldvalue {
@@ -713,14 +720,11 @@ sub _get_fieldvalue {
     open( my $fh, q{<}, $self->{'path'} ) or croak $OS_ERROR;
     while (<$fh>) {
         next if $INPUT_LINE_NUMBER == 1;    # skip column heading line
-        /^([^,]+)/msx;
+        /^([^\Q$self->{'delim'}\E]+)/msx;
         if ( $str eq $1 ) {
             chomp;    # or zipf will return with "\n" appended
-            my @line = split m/$self->{'delim'}/msx, $_;
-            $val = $line[$col_i];
-
-            #/^(([^,]+),){q{},$col_i}/g;
-            #$val = $1;
+            my @line = split m/\Q$self->{'delim'}\E/msx, $_;
+            $val = _nummify($line[$col_i]);
             last;
         }
     }
@@ -740,6 +744,12 @@ sub _in_range {
     return $res;
 }
 
+sub _nummify {
+    my $val = shift;
+    $val =~ s/,([^,]+)$/.$1/; # replace ultimate , with .
+    return $val;
+}
+
 =head1 DIAGNOSTICS
 
 =over 4
@@ -754,7 +764,11 @@ Croaked from L<new|Lingua::Norms::SUBTLEX/new> if called with a value for the ar
 
 =item Cannot find required database for language $lang
 
-Croaked from L<new|Lingua::Norms::SUBTLEX/new> if none of the given values to arguments B<lang>, B<dir> or B<path> are valid, and even the default site/lib directory and US database are not accessible. Check that your have indeed a file with the given value of B<lang> (US, UK or NL) within the Perl/site/lib/Lingua/Norms/SUBTLEX directory, or at least that the SUBTLEX-US file is located within it, and can be read via your script.
+Croaked from L<new|Lingua::Norms::SUBTLEX/new> if none of the given values to arguments B<lang>, B<dir> or B<path> are valid, and even the default site/lib directory and US database are not accessible. Check that your have indeed a file with the given value of B<lang> (DE, NL, UK or US) within the Perl/site/lib/Lingua/Norms/SUBTLEX directory, or at least that the SUBTLEX-US file is located within it, and can be read via your script.
+
+=item Cannot determine fields for given language
+
+Croaked upon construction if no fields are recognized for the given language. The value given to B<lang> must be one of DE, NL, UK or US.
 
 =item No string to test; pass a string to the function
 
@@ -788,6 +802,8 @@ L<Text::CSV::Separator|Text::CSV::Separator> : depended upon to determine the de
 
 =head1 REFERENCES
 
+B<Brysbaert, M., Buchmeier, M., Conrad, M., Jacobs, A.M., Boelte, J., & Boehl, A.> (2011). The word frequency effect: A review of recent developments and implications for the choice of frequency estimates in German. I<Experimental Psychology>, I<58>, 412-424. doi: L<10.1027/1618-3169/a000123|http://dx.doi.org/10.1027/1618-3169/a000123>
+
 B<Brysbaert, M., & New, B.> (2009). Moving beyond Kucera and Francis: A critical evaluation of current word frequency norms and the introduction of a new and improved word frequency measure for American English. I<Behavior Research Methods>, I<41>, 977-990. doi: L<10.3758/BRM.41.4.977|http://dx.doi.org/10.3758/BRM.41.4.977>
 
 B<Brysbaert, M., New, B., & Keuleers,E.> (2012). Adding part-of-speech information to the SUBTLEX-US word frequencies. I<Behavior Research Methods>, I<44>, 991-997. doi: L<10.3758/s13428-012-0190-4|http://dx.doi.org/10.3758/s13428-012-0190-4>
@@ -806,8 +822,8 @@ Roderick Garton, C<< <rgarton at cpan.org> >>
 
 =head1 BUGS AND LIMITATIONS
 
-Please report any bugs or feature requests to C<bug-lingua-norms-subtlfreq-0.04 at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Lingua-Norms-SUBTLEX-0.04>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-lingua-norms-subtlfreq-0.05 at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Lingua-Norms-SUBTLEX-0.05>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 =head1 SUPPORT
@@ -823,19 +839,19 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker (report bugs here)
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Lingua-Norms-SUBTLEX-0.04>
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Lingua-Norms-SUBTLEX-0.05>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
-L<http://annocpan.org/dist/Lingua-Norms-SUBTLEX-0.04>
+L<http://annocpan.org/dist/Lingua-Norms-SUBTLEX-0.05>
 
 =item * CPAN Ratings
 
-L<http://cpanratings.perl.org/d/Lingua-Norms-SUBTLEX-0.04>
+L<http://cpanratings.perl.org/d/Lingua-Norms-SUBTLEX-0.05>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/Lingua-Norms-SUBTLEX-0.04/>
+L<http://search.cpan.org/dist/Lingua-Norms-SUBTLEX-0.05/>
 
 =back
 
